#!/usr/bin/env python
#---------------------------------------------------------------------------
# Copyright 2021 Takafumi Ogawa
# Licensed under the Apache License, Version2.0.
#---------------------------------------------------------------------------
# Auxiliary tool of pydecs library for producing inpydecs_dos.csv file
#---------------------------------------------------------------------------
import os,sys
import datetime
import matplotlib.pyplot as plt

def convDOSfromVASP():
    print(" Preparing \"inpydecs_dos.csv\" from VASP-output files in the current directory.")
    print(" Parsed files: DOSCAR, OUTCAR, EIGENVAL")
    if not os.path.exists("OUTCAR"):
        print("ERROR::file_not_found: OUTCAR")
        sys.exit()
    if not os.path.exists("EIGENVAL"):
        print("ERROR::file_not_found: EIGANVAL")
        sys.exit()
    if not os.path.exists("DOSCAR"):
        print("ERROR::file_not_found: DOSCAR")
        sys.exit()


    fin=open("OUTCAR")
    l1=fin.readline()
    NEDOS=-1
    NIONS=-1
    ISPIN=-1
    LNONCOLLINEAR=-1
    natoms=[]
    elems=[]
    NBANDS=-1
    NKPTS=-1
    volume=-1
    while l1:
        if "number of dos" in l1:
            l2=l1.split()
            NEDOS=int(l2[5])
            NIONS=int(l2[-1])
        if "ISPIN" in l1:
            l2=l1.split()
            ISPIN=int(l2[2])
        if "LNONCOLLINEAR" in l1:
            l2=l1.split()
            LNONCOLLINEAR=l2[2]
        if "ions per type" in l1:
            l2=l1.split()
            for t2 in l2[4:]:
                natoms.append(int(t2))
        if "POSCAR =" in l1:
            l2=l1.split()
            for t2 in l2[2:]:
                elems.append(t2)
        if "NKPTS" in l1 and "NBANDS" in l1:
            l2=l1.split()
            NKPTS=int(l2[3])
            NBANDS=int(l2[-1])
        if "direct lattice vectors" in l1:
            l2=fin.readline().split()
            latt1=[float(t2) for t2 in l2[:3]]
            l2=fin.readline().split()
            latt2=[float(t2) for t2 in l2[:3]]
            l2=fin.readline().split()
            latt3=[float(t2) for t2 in l2[:3]]
            out23=[latt2[1]*latt3[2]-latt2[2]*latt3[1],
                   latt2[2]*latt3[0]-latt2[0]*latt3[2],
                   latt2[0]*latt3[1]-latt2[1]*latt3[0]]
            volume=out23[0]*latt1[0]+out23[1]*latt1[1]+out23[2]*latt1[2]
        l1=fin.readline()
    NSPIN=-1
    if LNONCOLLINEAR=="T":
        NSPIN=3
    if LNONCOLLINEAR=="F":
        NSPIN=ISPIN
    fin=open("EIGENVAL")
    for i1 in range(6):
        l1=fin.readline()
    E_VBM=-1.0e10
    E_CBM= 1.0e6
    for i1 in range(NKPTS):
        l1=fin.readline()
        l1=fin.readline()
        for i2 in range(NBANDS):
            l2=fin.readline().split()
            elist=[float(t2) for t2 in l2[1:1+NSPIN]]
            occlist=[float(t2) for t2 in l2[1+NSPIN:1+2*NSPIN]]
            for i3,occ3 in enumerate(occlist):
                if occ3 >0.5:
                    if E_VBM<elist[i3]:
                        E_VBM=elist[i3]
                elif occ3<0.1:
                    if E_CBM>elist[i3]:
                        E_CBM=elist[i3]
    E_gap=E_CBM-E_VBM
    fin=open("DOSCAR")
    for i1 in range(6):
        l1=fin.readline()
    enelist=[]
    doslist=[]
    l1=fin.readline()
    for i1 in range(NEDOS-1):
        l2=fin.readline().split()
        enelist.append(float(l2[0]))
        dsum=0.0
        for d1 in l2[1:1+NSPIN]:
            dsum+=float(d1)
        doslist.append(dsum)

    fout=open("inpydecs_dos.csv","w")
    fout.write("##############################\n")
    fout.write("# Produced by pydecs-aux-tool\n")
    dt1=str(datetime.datetime.today())
    fout.write("# Time = "+dt1[:dt1.rfind(":")]+"\n")
    fout.write("# ELEMENTS = ")
    for e1 in elems:
        fout.write(e1+" ")
    fout.write("\n")
    fout.write("# NATOMS = ")
    for n1 in natoms:
        fout.write(str(n1)+" ")
    fout.write("\n")
    fout.write("# NATOMStot = "+str(NIONS)+"\n")
    fout.write("# Volume = "+str(volume)+" [A^3]\n")
    fout.write("# Egap = "+str(E_gap)+" [eV]\n")
    fout.write("# EVBM = "+str(E_VBM)+" [eV]\n")
    fout.write("# NEDOS = "+str(NEDOS-1)+"\n")
    fout.write("##############################\n")
    fout.write("# energy,dos\n")
    for i2,e2 in enumerate(enelist):
        fout.write(str(e2)+","+str(doslist[i2])+"\n")
    fout.close()
    print(" Finished: Output filename = inpydecs_dos.csv") 
    print("-"*100)

    plt.figure(figsize=(7,6))
    plt.tick_params(axis="x",direction="in",top=True)
    plt.tick_params(axis="y",direction="in",right=True)
    plt.plot(enelist,doslist,"k-",linewidth=2.0)
    plt.xlim([enelist[0],enelist[-1]])
    #plt.ylim([min(charge_list)*1.1,max(charge_list)*1.1])
    plt.axvline(E_VBM,color="tab:red",ls="--",lw=1.5)
    plt.axvline(E_CBM,color="tab:red",ls="--",lw=1.5)
    plt.xlabel(r"$\mathrm{Energy\ [eV]}$",size=14)
    plt.ylabel(r"$\mathrm{Dinsity\ of\ states\ [/cell]}$",size=14)
    plt.savefig("inpydecs_dos.png",dpi=200,bbox_inches="tight")
    plt.close()


    plt.figure()
    plt.plot()

if __name__=="__main__":
    convDOSfromVASP()






#!/usr/bin/env python
#---------------------------------------------------------------------------
# Copyright 2021 Takafumi Ogawa
# Licensed under the Apache License, Version2.0.
#---------------------------------------------------------------------------
# pydecs-eqcond module
#---------------------------------------------------------------------------
import os,sys
import copy
import numpy as np
from pydecs.reference_phases import ReferencePhases
from pydecs.common import product_local

class EquilibriumConditions:

    def __init__(self,input_params,input_paths,root_outfiles="out"):
        if "output_details" in input_params.keys():
            output_details=input_params["output_details"]
        else:
            output_details=False
        if output_details:
            bool_silent_chempotcalc=False
        else:
            bool_silent_chempotcalc=True
        print(" Reading equilibrium conditions (eq)")
        fnout=root_outfiles+"_parameters_temp_gasP.csv"
        fout_tp=open(fnout,"w")
        ### constructing conditions
        temperatures_list=[]
        if "temperature" in input_params:
            temp_params=input_params["temperature"]
            if "tempK_min" in temp_params:
                temp_min=temp_params["tempK_min"]
            else:
                print(" ERROR(temperatures):: missed tempK_min")
                sys.exit()
            if "tempK_max" in temp_params:
                temp_max=temp_params["tempK_max"]
            else:
                temp_max=temp_min
            if "tempK_num" in temp_params:
                temp_num=temp_params["tempK_num"]
            else:
                temp_num=1
            temperatures_list=np.linspace(temp_min,temp_max,temp_num)
        if len(temperatures_list)!=0:
            str1="   Temperatures: "
            str_out="Temperatures,"
            for t1 in temperatures_list:
                str1+=f"{t1}, "
                str_out+=f"{t1}, "
            print(str1[:-2])
            fout_tp.write(str_out[:-2]+"\n")
        else:
            print(" ERROR(eq.temperature):: missed temperature")
            sys.exit()
        ###############################################
        ### Reading fix_Natoms
        self.elements_fix_Natoms=[]
        self.Natoms_list=[]
        self.Natoms_chempot_init=[]
        if "fix_Natoms" in input_params:
            params1=input_params["fix_Natoms"]
            for params2 in params1:
                e2=params2["element"]
                nat2_array=[]
                nat2_min="NONE"
                nat2_max="NONE"
                nat2_num=0
                nat2_log=False
                if "numCell_array" in params2:
                    nat2_array=params2["numCell_array"]
                if "numCell_min" in params2:
                    nat2_min=params2["numCell_min"]
                if "numCell_max" in params2:
                    nat2_max=params2["numCell_max"]
                if "numCell_num" in params2:
                    nat2_num=params2["numCell_num"]
                if "numCell_log" in params2:
                    nat2_log=params2["numCell_log"]
                if "chempot_init" in params2:
                    cpinit2=params2["chempot_init"]
                else:
                    print(" ERROR(eq.fix_Natoms):: missed chempot_init for "+e2)
                    sys.exit()
                if len(nat2_array)>0:
                    self.Natoms_list.append(np.array(nat2_array))
                else:
                    if nat2_num==0:
                        print(" ERROR(eq.fix_Natoms):: missed numCell_num for "+e2)
                        sys.exit()
                    if nat2_min=="NONE":
                        print(" ERROR(eq.fix_Natoms):: missed numCell_min for "+e2)
                        sys.exit()
                    if nat2_max=="NONE":
                        nat2_max=nat2_min
                    if nat2_log:
                        self.Natoms_list.append(np.logspace(np.log10(nat2_min),np.log10(nat2_max),nat2_num))
                    else:
                        self.Natoms_list.append(np.linspace(nat2_min,nat2_max,nat2_num))
                self.elements_fix_Natoms.append(e2)
                self.Natoms_chempot_init.append(cpinit2)
        if len(self.elements_fix_Natoms)!=0:
            for ie1,e1 in enumerate(self.elements_fix_Natoms):
                str1=f"   fix_Natoms({e1}): "
                for n1 in self.Natoms_list[ie1]:
                    str1+=f"{n1}, "
                print(str1[:-2])
        self.Natoms_linking = False
        self.Natoms_cp_scheme = "fix"
        if "fix_Natoms_params" in input_params:
            params1=input_params["fix_Natoms_params"]
            if "linking_multiple" in params1:
                self.Natoms_linking = params1["linking_multiple"]
        ###############################################
        self.elements_fix_chempots=[]
        self.chempots_list=[]
        cond_fixchempots_list=[]
        ### reading common-data
        if "fix_chempots" in input_params:
            params1=input_params["fix_chempots"]
            if "elements" in params1:
                self.elements_fix_chempots=params1["elements"]
        if len(self.elements_fix_chempots)>0:
            params1=input_params["fix_chempots"]
            for e1 in self.elements_fix_chempots:
                for e2 in self.elements_fix_Natoms:
                    if e1==e2:
                        print(f" ERROR(eq):: duplicate elements in fix_chempots and fix_Natoms: {e1}")
                        sys.exit()
            if "lambda_del" in params1:
                lambda_del=params1["lambda_del"]
            else:
                lambda_del=0.1
            str1="   Elements for fix-chempots: "
            for e1 in self.elements_fix_chempots:
                str1+=f"{e1}, "
            print(str1[:-2])
            print("-"*50)
            refPhase=ReferencePhases(self.elements_fix_chempots,input_paths)
            if not "eq_phases" in params1:
                print(" ERROR(eq.fix_chempots.eq_phases):: missed")
                sys.exit()
            ### reading eq_phases
            phases_whole_range=[]
            phases_limits={}
            for params2 in params1["eq_phases"]:
                if not "label" in params2:
                    print(" ERROR(eq.fix_chempots.eq_phases):: missed label")
                    sys.exit()
                l1=params2["label"]
                if not "phases" in params2:
                    print(" ERROR(eq.fix_chempots.eq_phases):: missed phases")
                    sys.exit()
                if l1=="whole-range":
                    phases_whole_range=params2["phases"]
                else:
                    phases_limits[l1]=params2["phases"]
            if len(phases_limits)==0:
                phases_limits["whole"]=phases_whole_range
            else:
                for l1,ph_list in phases_limits.items():
                    ph_list.extend(phases_whole_range)
            if len(phases_limits)==1:
                for l1,ph_list in phases_limits.items():
                    phases_limits={"whole":ph_list}
            for l1,ph_list in phases_limits.items():
                str1=f"   Eq-phases ({l1}): "
                for ph1 in ph_list:
                    str1+=f"{ph1}, "
                    if not refPhase.exists(ph1):
                        print(f" ERROR(eq.fix_chempots):: phase not found: {ph1}")
                        sys.exit()
                print(str1[:-2])
                if len(ph_list)!=len(self.elements_fix_chempots):
                    print(f" ERROR(eq.fix_chempots):: #eq_phases({l1}) is not equal to #elements(fix_chempots)")
                    sys.exit()
            ### reading gas_pressures
            gas_pressures={}
            if "gas_pressures" in params1:
                for params2 in params1["gas_pressures"]:
                    if not "label" in params2:
                        print(" ERROR(eq.fix_chempots.gas_pressures):: missed label")
                        sys.exit()
                    if not "pressPa_min" in params2:
                        print(" ERROR(eq.fix_chempots.gas_pressures):: missed pressPa_min")
                        sys.exit()
                    if "scale_log" in params2:
                        scale_log=params2["scale_log"]
                    else:
                        scale_log=True
                    l1=params2["label"]
                    pmin=params2["pressPa_min"]
                    if "pressPa_max" in params2:
                        pmax=params2["pressPa_max"]
                    else:
                        pmax=pmin
                    if "pressPa_num" in params2:
                        pnum=params2["pressPa_num"]
                    else:
                        pnum=1
                    bool_label=False
                    for l2 in phases_whole_range:
                        if l1==l2:
                            bool_label=True
                    for ph1 in phases_limits.values():
                        for l2 in ph1:
                            if l1==l2:
                                bool_label=True
                    if bool_label:
                        if scale_log:
                            gas_pressures[l1]=np.logspace(np.log10(pmin),np.log10(pmax),pnum)
                        else:
                            gas_pressures[l1]=np.linspace(pmin,pmax,pnum)
            for l1,p1_list in gas_pressures.items():
                str1=f"   Pressures ({l1}): "
                str_out=f"Pressures_{l1}, "
                for p1 in p1_list:
                    str1+=f"{p1:7.1e}, "
                    str_out+=f"{p1:20.15f}, "
                print(str1[:-2])
                fout_tp.write(str_out[:-2]+"\n")
            if len(temperatures_list)==0:
                temp_press_prodlist=["NONE"]
                temp_press_params=[]
            else:
                temp_press_prodlist=temperatures_list
                temp_press_params=["T"]
            for l1,p1_list in gas_pressures.items():
                temp_press_prodlist=product_local(temp_press_prodlist,p1_list)
                temp_press_params.append("P_"+l1)
            if isinstance(temp_press_prodlist[0],float):
                tmp1=[]
                for t1 in temp_press_prodlist:
                    tmp1.append([t1])
                temp_press_prodlist=tmp1
            temp_press_list=[]
            for t1 in temp_press_prodlist:
                d1={}
                for it2,t2 in enumerate(t1):
                    d1[temp_press_params[it2]]=t2
                temp_press_list.append(d1)
            fout_tp.close()
            #### Constructing lambda-list
            lambda_list=[]
            if len(phases_limits)==1:
                lambda_list=[{"whole":1.0}]
            elif len(phases_limits)>1:
                lambda_prodlist=[[t1] for t1 in np.arange(0.0,1.0+1.0e-10,lambda_del)]
                lambda_params=[]
                for i1 in range(len(phases_limits)-2):
                    t1=np.arange(0.0,1.0,lambda_del)
                    lambda_prodlist=product_local(lambda_prodlist,t1)
                for l1 in phases_limits.keys():
                    lambda_params.append(l1)
                for ilam1,lam1 in enumerate(lambda_prodlist):
                    d1={}
                    sum2=0.0
                    for it2,lam2 in enumerate(lam1):
                        sum2+=lam2
                    if sum2>1.0:
                        continue
                    lam1.append(1.0-sum2)
                    for it2,lam2 in enumerate(lam1):
                        d1[lambda_params[it2]]=lam2
                    lambda_list.append(d1)
            if len(phases_limits)>0:
                fnout_lam=root_outfiles+"_parameters_lambda.csv"
                fout_lam=open(fnout_lam,"w")
                str_out=""
                for l1 in lambda_list[0].keys():
                    str_out+=f"{l1:^20s}, "
                fout_lam.write(str_out[:-2]+"\n")
                for lam1 in lambda_list:
                    str_out=""
                    for l1,v1 in lam1.items():
                        str_out+=f"{v1:20.15f}, "
                    fout_lam.write(str_out[:-2]+"\n")
                fout_lam.close()
            def make_ph_list(phlist_in):
                phlist_out=[]
                for ph1 in phlist_in:
                    if refPhase.issolid(ph1):
                        phlist_out.append((ph1,0.0))
                    else:
                        for k1,v1 in tp1.items():
                            if k1=="T":
                                continue
                            if k1.split("_")[1].strip()==ph1:
                                phlist_out.append((ph1,v1))
                return phlist_out
            ### update cond_fixchempots_list
            for tp1 in temp_press_list:
                cp_limits={}
                for l1,ph_list in phases_limits.items():
                    phin_list=make_ph_list(ph_list)
                    cp1=refPhase.calc_atomChempots(tp1["T"],phin_list,bool_silent_chempotcalc)
                    cp_limits[l1]=cp1
                for lam1 in lambda_list:
                    cp1={}
                    for e1 in self.elements_fix_chempots:
                        cp2=0.0
                        for l2,coeff2 in lam1.items():
                            cp2+=coeff2*cp_limits[l2][e1]
                        cp1[e1]=cp2
                    tp2=copy.deepcopy(tp1)
                    for e1,cp2 in cp1.items():
                        tp2["chempot_"+e1]=cp2
                    for l2,coeff2 in lam1.items():
                        tp2["lambda_"+l2]=coeff2
                    cond_fixchempots_list.append(tp2)
        if len(cond_fixchempots_list)==0:
            for tp1 in temperatures_list:
                cond_fixchempots_list.append({"T":tp1})
        #################################################
        ### Constructing fix-Natoms!!!
        Natoms_target_prod_list=[]
        if len(self.elements_fix_Natoms)>0:
            fnout=root_outfiles+"_parameters_fixNatoms.csv"
            fout_nat=open(fnout,"w")
            str1=""
            for ie1,e1 in enumerate(self.elements_fix_Natoms):
                str1+=f"{e1} , "
            fout_nat.write(str1[:-2]+"\n")
            if len(self.elements_fix_Natoms)==1:
                Natoms_target_prod_list=[[t1] for t1 in self.Natoms_list[0]]
            else:
                if self.Natoms_linking:
                    len1=len(self.Natoms_list[0])
                    for i1 in range(1,len(self.Natoms_list)):
                        len2=len(self.Natoms_list[i1])
                        if len1!=len2:
                            print(" ERROR(eq.fix_Natoms_params):: linking_multiple-param is set in the input toml-file, but the length of input Natoms values in [[fix_Natoms]] are not coincident!")
                            sys.exit()
                    for i1 in range(len1):
                        tmp_target=[]
                        for ie1 in range(len(self.Natoms_list)):
                            tmp_target.append(self.Natoms_list[ie1][i1])
                        Natoms_target_prod_list.append(tmp_target)
                else:
                    Natoms_target_prod_list=copy.deepcopy(self.Natoms_list[0])
                    for ie1,e1 in enumerate(self.elements_fix_Natoms):
                        if ie1==0:
                            continue
                        Natoms_target_prod_list=product_local(Natoms_target_prod_list,self.Natoms_list[ie1])
            for nlist1 in Natoms_target_prod_list:
                str1=""
                for n1 in nlist1:
                    str1+=f"{n1} , "
                fout_nat.write(str1[:-2]+"\n")
            fout_nat.close()
        #################################################
        ### Combining the parameters of fix-chempots and fix-Natoms!!!
        self.eq_conditions=[]
        if len(self.elements_fix_Natoms)==0:
            self.eq_conditions=cond_fixchempots_list
        else:
            for nlist1 in Natoms_target_prod_list:
                cond3_fixNatoms=[]
                for ie1,e1 in enumerate(self.elements_fix_Natoms):
                    cond4={}
                    cond4["element"]=e1
                    cond4["target_Natoms"]=nlist1[ie1]
                    cond4["chempot_init"]=self.Natoms_chempot_init[ie1]
                    cond3_fixNatoms.append(cond4)
                if self.Natoms_linking:
                    cond5={"fix_Natoms_linked":cond3_fixNatoms}
                    # cond5={"fix_Natoms":cond3_fixNatoms}
                else:
                    cond5={"fix_Natoms":cond3_fixNatoms}
                for cond1 in cond_fixchempots_list:
                    cond2=copy.deepcopy(cond5)
                    cond2.update(cond1)
                    self.eq_conditions.append(cond2)
        print("-"*100)
        self.id_cond=0

    def __iter__(self):
        return self

    def __next__(self):
        if self.id_cond == len(self.eq_conditions):
            raise StopIteration()
        cond_out=self.eq_conditions[self.id_cond]
        self.id_cond+=1
        return cond_out

    def get_eq_conditions(self):
        return self.eq_conditions

    def get_elements(self):
        return self.elements_fix_chempots+self.elements_fix_Natoms




#!/usr/bin/env python
#-----------------------------------------------------------------------------
# Copyright 2021 Takafumi Ogawa
# Licensed under the Apache License, Version2.0.
#-----------------------------------------------------------------------------
# Auxiliary tool of pydecs library for checking equilibrium phases
#-----------------------------------------------------------------------------
import os,sys
import numpy as np
from scipy import interpolate

class ReferencePhases:

    def parse_composition(self,comp_in):
        comp1=[]
        ic0=0
        for ic1,c1 in enumerate(comp_in):
            if ic1!=0 and c1.isupper():
                comp1.append(comp_in[ic0:ic1])
                ic0=ic1
        comp1.append(comp_in[ic0:])
        atomList={}
        for c1 in comp1:
            elem1=""
            num1=""
            for c2 in c1:
                if c2.isdigit():
                    num1+=c2
                else:
                    elem1+=c2
            if len(num1)==0:
                num1="1"
            atomList[elem1]=int(num1)
        return atomList


    
    def check_host(self,phaseList_in):
        self.phase_host=""
        for ph1 in phaseList_in:
            if ph1["type"]=="solid*":
                self.phase_host=ph1
        if self.phase_host=="":
            print(" WARNING::host-tag not-found (inpydecs_phases.csv): solid*")
        print("   Host-composition: "+self.phase_host["composition"])
        return 

    def get_host_composition(self):
        return self.phase_host["composition"]

    def prepare_finterp(self):
        for ph1 in self.phaseList:
            fn1=ph1["filename_delG"]
            if len(fn1)==0:
                ph1["delG_finterp"]="NONE"
                continue
            fin=open(self.filename_root+fn1)
            xlist=[]
            ylist=[]
            l1 = fin.readline()
            while l1:
                if l1.strip()[0]!="#":
                    l2=l1.split(",")
                    xlist.append(float(l2[0]))
                    ylist.append(float(l2[1]))
                l1 = fin.readline()
            finterp=interpolate.interp1d(xlist,ylist,kind="cubic")
            ph1["delG_finterp"]=finterp

    def parse_phases(self,filename_phases_in="inpydecs_phases.csv",elemsList_in="host"):
        if not os.path.exists(filename_phases_in):
            print(" ERROR::file_not_found: "+filename_phases_in)
            sys.exit()
        print(" Reading file: "+filename_phases_in)
        fin=open(filename_phases_in).readlines()
        columns=[ t1.strip() for t1 in fin[0].split(",")]
        column_names=set(["commentout","type","composition","energy_0K","filename_delG"])
        for c1 in column_names:
            if not c1 in columns:
                print(" ERROR::column_not_found: "+c1)
                sys.exit()
        phaseList0=[]
        for l1 in fin[1:]:
            l2=[ t1.strip() for t1 in l1.split(",")]
            ph0={}
            for i3,l3 in enumerate(l2):
                c3=columns[i3]
                if c3=="commentout":
                    ph0[c3]=l3
                if c3=="filename_delG":
                    ph0[c3]=l3
                if c3=="composition":
                    ph0[c3]=l3
                    ph0["composition_dict"]=self.parse_composition(l3)
                if c3=="type":
                    ph0[c3]=l3
                if c3=="energy_0K":
                    ph0[c3]=l3
            if not "composition" in ph0.keys()\
                or not "composition_dict" in ph0.keys()\
                or not "energy_0K" in ph0.keys()\
                or not "type" in ph0.keys():
                continue
            if len(ph0["commentout"])==0 and len(ph0["composition"])>0:
                if not ph0["type"] in ["gas","solid","solid*"]:
                   print(" WARNING::invalid type: "+(l3))
                ph0["energy_0K"]=float(ph0["energy_0K"])
                phaseList0.append(ph0)
        self.check_host(phaseList0)
        comp_host=self.phase_host["composition"]
        comp_host_elems=set(self.phase_host["composition_dict"].keys())
        if elemsList_in=="host":
            slected_elems=comp_host_elems
        else:
            slected_elems=set(elemsList_in)
        phaseList1=[]
        for ph1 in phaseList0:
            comp1_elems=set(ph1["composition_dict"].keys())
            elems_sum=slected_elems.union(comp1_elems)
            if len(elems_sum)<=len(slected_elems):
                phaseList1.append(ph1)
        self.phaseList=phaseList1
        print("-"*50)

    def get_phaseList(self):
        return self.phaseList

    def __init__(self,elemsList_in="host",input_paths=["./"]):
        fnin="NONE"
        for path1 in input_paths:
            if os.path.exists(path1+"inpydecs_phases.csv"):
                fnin=path1+"inpydecs_phases.csv"
                break
        if fnin=="NONE":
            print(" ERROR::file not-found: inpydecs_phases.csv")
            sys.exit()
        if "/" in fnin:
            self.filename_root=fnin[:fnin.rfind("/")+1]
        else:
            self.filename_root=""
        self.parse_phases(fnin,elemsList_in)

    def get_phasedata(self,comp_in):
        ph_target="NONE"
        for ph1 in self.phaseList:
            if ph1["composition"]==comp_in:
                ph_target=ph1
                return ph_target
        if ph_target=="NONE":
            print(" WARNING::phasedata not-found for compound: "+comp_in)
        return ph_target

    def get_singleFreeEnergy(self,comp_in,temp_in,press_in):
        ph1=self.get_phasedata(comp_in)
        energy1=ph1["energy_0K"]
        if "delG_finterp" not in ph1.keys():
            self.prepare_finterp()
        if ph1["delG_finterp"]!="NONE":
            energy1+=ph1["delG_finterp"](temp_in)
        if ph1["type"]=="gas":
            energy1+=(8.61733262e-5)*temp_in*np.log(press_in/1.0e5)
        return energy1
    
    def calc_atomChempots(self,temp_in,comp_press_list_in,bool_silent=False):
        if not bool_silent:
            print(" Calculating chemical potentials")
            print("   Temperature = "+str(temp_in))
            for (c1,p1) in comp_press_list_in:
                print("   Compound = "+str(c1)+" ; Presssure = "+str(p1))
        elems=[]
        atomnumsList=[]
        eneList=[]
        for (c1,p1) in comp_press_list_in:
            ph1=self.get_phasedata(c1)
            ene1=self.get_singleFreeEnergy(c1,temp_in,p1)
            c2=ph1["composition_dict"]
            atomnums2={}
            for e1,n1 in c2.items():
                if e1 not in elems:
                    elems.append(e1)
                atomnums2[e1]=float(n1)
            atomnumsList.append(atomnums2)
            eneList.append([ene1])
        coeffsList=[]
        if not bool_silent:
            print(" Constructed simultaneous linear equations")
        for iat1,at1 in enumerate(atomnumsList):
            coeffs=np.zeros(len(elems))
            for ie1,e1 in enumerate(elems):
                if e1 in at1.keys():
                    coeffs[ie1]=at1[e1]
            coeffsList.append(coeffs)
            str1="   "
            for ie1,e1 in enumerate(elems):
                str1+=str(coeffs[ie1])+" mu_"+e1+" + "
            str1=str1[:-3]
            str1+=" = "+str(eneList[iat1][0])
            if not bool_silent:
                print(str1)
        if len(comp_press_list_in)<len(elems):
            print(" ERROR:: The above equations cannot be solved.")
            sys.exit()
        coeffsList=np.matrix(coeffsList)
        coeffsList_inv=np.linalg.inv(coeffsList)
        muList1=coeffsList_inv*np.matrix(eneList)
        if not bool_silent:
            print(" Solved results")
        muList2={}
        for ie1,e1 in enumerate(elems):
            muList2[e1]=float(muList1[ie1][0])
        for ie1,e1 in enumerate(elems):
            str1="   mu_"+e1+" = "+str(muList2[e1])
            if not bool_silent:
                print(str1)
        if not bool_silent:
            print("-"*50)
        return muList2

    def exists(self,phase_in):
        t1=self.get_phasedata(phase_in)
        if t1=="NONE":
            return False
        return True

    def issolid(self,phase_in):
        t1=self.get_phasedata(phase_in)
        if "solid" in t1["type"]:
            return True
        return False
